<?php
$phpver = phpversion();
$date = date('Y-m-d');
$filename = "versions/{$date}-php-{$phpver}.json";

$all_extensions = get_loaded_extensions();
$all_extensions = array_merge($all_extensions, get_unpacked_extensions());
sort($all_extensions);

$extensions_data = [];
foreach ($all_extensions as $ext) {
    $extensions_data[] = [
        'name' => $ext,
        'loaded' => in_array($ext, get_loaded_extensions())
    ];
}

$data = [
    'php_version' => $phpver,
    'extensions' => $extensions_data
];

// Create the "versions" directory if it doesn't exist
if (!is_dir('versions')) {
    mkdir('versions', 0755, true);
}

// Write the JSON data to the file
file_put_contents($filename, json_encode($data, JSON_PRETTY_PRINT));

// Set the appropriate headers for file download
header('Content-Type: application/json');
header('Content-Disposition: attachment; filename="' . basename($filename) . '"');
header('Expires: 0');
header('Cache-Control: must-revalidate');
header('Pragma: public');
header('Content-Length: ' . filesize($filename));

// Output the file contents
readfile($filename);
exit;
?>

<table>
    <tr>
        <th colspan="2">PHP Version: <?php echo $phpver; ?></th>
    </tr>
    <tr>
        <th>Server Extensions</th>
    </tr>
    <?php
    foreach ($extensions_data as $ext_data) {
        echo '<tr>';
        if ($ext_data['loaded']) {
            echo '<td>☑ ' . $ext_data['name'] . '</td>';
        } else {
            echo '<td>☐ ' . $ext_data['name'] . '</td>';
        }
        echo '</tr>';
    }

    function get_unpacked_extensions() {
        $extension_dir = ini_get('extension_dir');
        $extensions = array();

        if ($extension_dir) {
            $dir = new DirectoryIterator($extension_dir);
            foreach ($dir as $fileinfo) {
                if ($fileinfo->isFile() && pathinfo($fileinfo->getFilename(), PATHINFO_EXTENSION) === 'so') {
                    $extensions[] = pathinfo($fileinfo->getFilename(), PATHINFO_FILENAME);
                }
            }
        }

        return $extensions;
    }
    ?>
</table>

<button onclick="downloadAsJSON()">Download as JSON</button>

<script>
function downloadAsJSON() {
    const currentDate = new Date();
    const fileName = `${currentDate.getFullYear()}-${String(currentDate.getMonth() + 1).padStart(2, '0')}-${String(currentDate.getDate()).padStart(2, '0')}-${phpversion()}.json`;
    const data = {
        phpVersion: phpversion(),
        loadedExtensions: <?php echo json_encode(get_loaded_extensions()); ?>,
        unpackedExtensions: <?php echo json_encode(get_unpacked_extensions()); ?>
    };

    const jsonData = JSON.stringify(data);
    const blob = new Blob([jsonData], { type: 'application/json' });
    const url = URL.createObjectURL(blob);

    const link = document.createElement('a');
    link.href = url;
    link.setAttribute('download', fileName);
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
}

function phpversion() {
    return '<?php echo phpversion(); ?>';
}
</script>
