<!DOCTYPE html>
<html>
<head>
    <title>Game Sales Data</title>
    <style>
        table {
            width: 100%;
            border-collapse: collapse;
        }

        th, td {
            padding: 8px;
            text-align: left;
            border-bottom: 1px solid #ddd;
        }

        th {
            background-color: #f2f2f2;
        }

        img {
            max-width: 100px;
            height: auto;
        }
    </style>
    <script src="https://cdn.jsdelivr.net/npm/chart.js"></script>
</head>
<body>
    <h1>Game Sales Data</h1>

    <table>
        <thead>
            <tr>
                <th>Position</th>
                <th>Platform</th>
                <th>Title</th>
                <th>This Week</th>
                <th>Cumulative</th>
                <th>Sell-through</th>
                <th>Image</th>
            </tr>
        </thead>
        <tbody>
            <?php
            $url = "https://www.famitsu.com/ranking/game-sales/";
            $headers = array(
                "User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/113.0",
                "Referer: https://www.famitsu.com/"
            );

            $context = stream_context_create(array(
                'http' => array(
                    'method' => 'GET',
                    'header' => implode("\r\n", $headers)
                )
            ));

            $html = file_get_contents($url, false, $context);

            if ($html === false) {
                echo "Error: Unable to fetch the website content.";
                return;
            }

            $dom = new DOMDocument();
            @$dom->loadHTML($html);

            $xpath = new DOMXPath($dom);
            $elements = $xpath->query('//div[@class="card card-game-sale-rank card-game-sale-rank--col-12 card-game-sale-rank--col-sm-12 card-game-sale-rank--col-md-12 card-game-sale-rank--col-lg-8"]');

            $gameData = [];
            $labels = [];
            $data = [];
            $platforms = [];

            $position = 1;
            foreach ($elements as $element) {
                $platform_elements = $xpath->query('.//span[contains(@class, "icon-console")]', $element);
                $platform = "";
                foreach ($platform_elements as $platform_element) {
                    $platform_class = $platform_element->getAttribute('class');
                    if (preg_match('/icon-console--bg-(\w+)/', $platform_class, $matches)) {
                        $platform = $matches[1];
                        if (!in_array($platform, $platforms)) {
                            $platforms[] = $platform;
                        }
                        break;
                    }
                }

                $title = $xpath->query('.//div[@class="card-game-sale-rank__title"]', $element)->item(0)->textContent;
                $this_week = $xpath->query('.//p[@class="card-game-sale-rank__sales-num-past card-game-sale-rank__sales-num"]', $element)->item(0)->textContent;
                $cumulative = $xpath->query('.//p[@class="card-game-sale-rank__sales-num-total card-game-sale-rank__sales-num"]', $element)->item(0)->textContent;
                $price = $xpath->query('.//span[@class="card-game-sale-rank__sales-meter-num"]', $element)->item(0)->textContent;
                $image_url = $xpath->query('.//div[@class="media-image media-image--game-platform media-image--' . $platform . ' media-image--square"]', $element)->item(0)->getAttribute('style');
                $image_url = preg_replace('/^background-image: url\("?([^"]+)"?\);\s*$/', '$1', $image_url);

                echo "<tr>";
                echo "<td>" . $position . "</td>";
                echo "<td>" . $platform . "</td>";
                echo "<td>" . $title . "</td>";
                echo "<td>" . $this_week . "</td>";
                echo "<td>" . $cumulative . "</td>";
                echo "<td>" . $price . "</td>";
                echo "<td><img src='" . $image_url . "' alt='" . $title . "'></td>";
                echo "</tr>";

                $gameData[] = [
                    'Position' => $position,
                    'Platform' => $platform,
                    'Title' => $title,
                    'ThisWeek' => (int) str_replace(',', '', $this_week),
                    'Cumulative' => (int) str_replace(',', '', $cumulative),
                    'SellThrough' => $price,
                    'Image' => $image_url
                ];

                $labels[] = $title;
                $data[] = (int) str_replace(',', '', $this_week);

                $position++;
            }

            $platformOptions = "";
            foreach ($platforms as $platform) {
                $platformOptions .= "<option value='" . $platform . "'>" . ucfirst($platform) . "</option>";
            }
            ?>
        </tbody>
    </table>

    <canvas id="gameChart"></canvas>
    <button id="downloadButton">Download Chart Data</button>

    <script>
        var ctx = document.getElementById('gameChart').getContext('2d');
        var gameChart = new Chart(ctx, {
            type: 'bar',
            data: {
                labels: <?php echo json_encode($labels); ?>,
                datasets: [{
                    label: 'This Week\'s Sales',
                    data: <?php echo json_encode($data); ?>,
                    backgroundColor: 'rgba(54, 162, 235, 0.2)',
                    borderColor: 'rgba(54, 162, 235, 1)',
                    borderWidth: 1
                }]
            },
            options: {
                scales: {
                    y: {
                        beginAtZero: true
                    }
                }
            }
        });

        // Add a platform filter dropdown
        var platformFilter = document.createElement("div");
        platformFilter.innerHTML = "<label for='platformSelect'>Filter by Platform:</label> <select id='platformSelect'><option value=''>All</option><?php echo $platformOptions; ?></select>";
        document.body.appendChild(platformFilter);

        // Add event listener to the platform filter dropdown
        var platformSelect = document.getElementById("platformSelect");
        platformSelect.addEventListener("change", function() {
            var selectedPlatform = this.value;
            var rows = document.querySelectorAll("table tbody tr");
            rows.forEach(function(row) {
                var platformCell = row.getElementsByTagName("td")[1];
                if (selectedPlatform === "" || platformCell.textContent === selectedPlatform) {
                    row.style.display = "table-row";
                } else {
                    row.style.display = "none";
                }
            });
        });

        // Function to download chart data as JSON
        function downloadChartData() {
            var gameData = <?php echo json_encode($gameData); ?>;
            var json = JSON.stringify(gameData, null, 2);
            var blob = new Blob([json], { type: "application/json" });
            var url = URL.createObjectURL(blob);
            var a = document.createElement("a");
            var date = new Date().toISOString().slice(0, 10); // yyyy-mm-dd format
            a.href = url;
            a.download = date + "-chart.json";
            document.body.appendChild(a);
            a.click();
            document.body.removeChild(a);
        }

        // Add event listener to the download button
        document.getElementById("downloadButton").addEventListener("click", downloadChartData);
    </script>
</body>
</html>
