<?php
$extractPath = '';
$zeroByteFiles = [];

function deleteDirectory($dir) {
  if (!file_exists($dir)) {
    return;
  }

  $files = array_diff(scandir($dir), array('.', '..'));

  foreach ($files as $file) {
    $path = $dir . '/' . $file;

    if (is_dir($path)) {
      deleteDirectory($path);
    } else {
      unlink($path);
    }
  }

  rmdir($dir);
}

if ($_SERVER["REQUEST_METHOD"] === "POST") {
  $allowedExtensions = ['zip'];

  // Check if a file was uploaded
  if (!empty($_FILES['zipFile']['name'])) {
    $filename = $_FILES['zipFile']['name'];
    $fileExtension = pathinfo($filename, PATHINFO_EXTENSION);

    if (!in_array($fileExtension, $allowedExtensions)) {
      echo "Error: Only zip files are allowed.";
      exit;
    }

    $targetFilePath = $filename;

    if (move_uploaded_file($_FILES['zipFile']['tmp_name'], $targetFilePath)) {
      $extractPath = pathinfo($filename, PATHINFO_FILENAME) . '/';

      $zip = new ZipArchive;
      if ($zip->open($targetFilePath) === true) {
        $zip->extractTo($extractPath);
        $zip->close();

        // Loop through extracted files
        $extractedFiles = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($extractPath));
        foreach ($extractedFiles as $file) {
          if ($file->isFile() && $file->getSize() === 0) {
            $zeroByteFiles[] = $file->getPathname();
          }
        }

        // Unlink the extracted folder
        deleteDirectory($extractPath);
      } else {
        echo "Error: Failed to open the zip file.";
      }
    } else {
      echo "Error: Failed to upload the zip file.";
    }
  } elseif (!empty($_POST['zipUrl'])) {
    $zipUrl = $_POST['zipUrl'];
    $filename = basename($zipUrl);

    $fileExtension = pathinfo($filename, PATHINFO_EXTENSION);

    if (!in_array($fileExtension, $allowedExtensions)) {
      echo "Error: Only zip files are allowed.";
      exit;
    }

    $targetFilePath = $filename;

    if (copy($zipUrl, $targetFilePath)) {
      $extractPath = pathinfo($filename, PATHINFO_FILENAME) . '/';

      $zip = new ZipArchive;
      if ($zip->open($targetFilePath) === true) {
        $zip->extractTo($extractPath);
        $zip->close();

        // Loop through extracted files
        $extractedFiles = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($extractPath));
        foreach ($extractedFiles as $file) {
          if ($file->isFile() && $file->getSize() === 0) {
            $zeroByteFiles[] = $file->getPathname();
          }
        }

        // Unlink the extracted folder
        deleteDirectory($extractPath);
      } else {
        echo "Error: Failed to open the zip file.";
      }
    } else {
      echo "Error: Failed to download the zip file from the URL.";
    }
  }
}
?>

<!DOCTYPE html>
<html>
<head>
  <title>Zip File Upload and Extraction</title>
</head>
<body>
  <h1>Zip File Upload and Extraction</h1>
  <form method="POST" enctype="multipart/form-data">
    <p>
      <label for="zipFile">Upload Zip File:</label>
      <input type="file" id="zipFile" name="zipFile" accept=".zip">
    </p>
    <p>
      <label for="zipUrl">or Enter Zip File URL:</label>
      <input type="text" id="zipUrl" name="zipUrl" value="https://alcea-wisteria.de/PHP/0demo/php.zip">
    </p>
    <p>
      <button type="submit">Process</button>
    </p>
  </form>

  <?php if (!empty($zeroByteFiles)) { ?>
    <h2>Zero-Byte Files:</h2>
    <ul>
      <?php foreach ($zeroByteFiles as $filepath) { ?>
        <li><?php echo $filepath; ?></li>
      <?php } ?>
    </ul>
  <?php } ?>
</body>
</html>